from daipecore.lineage.argument.DecoratorInputFunctionInterface import DecoratorInputFunctionInterface


class FileRead(DecoratorInputFunctionInterface):
    def __init__(self, path: str):
        self.__path = path

    @property
    def path(self):
        return self.__path



from daipecore.decorator.DecoratedDecorator import DecoratedDecorator
from pysparkbundle.write.PathWriterDecorator import PathWriterDecorator


@DecoratedDecorator
class csv_append(PathWriterDecorator):  # noqa: N801
    _mode = "append"
    _writer_service = "pysparkbundle.write.PathWriter.csv"



from pyspark.sql.session import SparkSession
from pyspark.sql.dataframe import DataFrame
from pysparkbundle.jdbc.OptionsFactoryInterface import OptionsFactoryInterface


class TableReader:
    def __init__(
        self,
        options_factory: OptionsFactoryInterface,
        spark: SparkSession,
    ):
        self.__options_factory = options_factory
        self.__spark = spark

    def read(self, table_name: str, **kwargs) -> DataFrame:
        options = {**self.__options_factory.create(), **kwargs}

        return self.__spark.read.format("jdbc").options(**options).option("dbtable", table_name).load()



from daipecore.decorator.DecoratedDecorator import DecoratedDecorator
from pysparkbundle.write.PathWriterDecorator import PathWriterDecorator


@DecoratedDecorator
class parquet_overwrite(PathWriterDecorator):  # noqa: N801
    _mode = "overwrite"
    _writer_service = "pysparkbundle.PathWriter.parquet"



from injecta.container.ContainerInterface import ContainerInterface
from pyspark.sql.types import StructType
from daipecore.function.input_decorator_function import input_decorator_function
from pysparkbundle.read.PathReader import PathReader


@input_decorator_function
def read_json(path: str, schema: StructType = None, options: dict = None):
    def wrapper(container: ContainerInterface):
        path_reader: PathReader = container.get("pysparkbundle.read.PathReader.json")

        return path_reader.read(path, schema, options)

    return wrapper



from logging import Logger
from pyspark.sql import SparkSession
from pyspark.sql.types import StructType


class PathReader:
    def __init__(self, format_: str, logger: Logger, spark: SparkSession):
        self.__format = format_
        self.__logger = logger
        self.__spark = spark

    def read(self, path: str, schema: StructType = None, options: dict = None):
        self.__logger.info(f"Reading {self.__format} from `{path}`", extra={"options": options})

        data_frame_reader = self.__spark.read.format(self.__format)

        if schema:
            data_frame_reader = data_frame_reader.schema(schema)

        if options:
            data_frame_reader = data_frame_reader.options(**options)

        return data_frame_reader.load(path)



from typing import Union
from pysparkbundle.write.PathWriter import PathWriter
from daipecore.decorator.OutputDecorator import OutputDecorator
from injecta.container.ContainerInterface import ContainerInterface
from pyspark.sql import DataFrame


class PathWriterDecorator(OutputDecorator):  # noqa: N801

    _mode: str
    _writer_service: str

    def __init__(self, path: str, partition_by: Union[str, list] = None, options: dict = None):
        self.__path = path

        if partition_by is None:
            self.__partition_by = []
        elif isinstance(partition_by, str):
            self.__partition_by = [partition_by]
        elif isinstance(partition_by, list):
            self.__partition_by = partition_by
        else:
            raise Exception(f"Unexpected partition_by type: {type(partition_by)}")

        self.__options = options

    def process_result(self, result: DataFrame, container: ContainerInterface):
        path_writer: PathWriter = container.get(self._writer_service)
        path_writer.write(result, self.__path, self._mode, self.__partition_by, self.__options)



from daipecore.lineage.argument.ArgumentMappingInterface import ArgumentMappingInterface
from pysparkbundle.csv.lineage.CsvRead import CsvRead
from pysparkbundle.json.lineage.JsonRead import JsonRead


class ArgumentMapping(ArgumentMappingInterface):
    def get_mapping(self):
        return {
            "read_csv": CsvRead,
            "read_json": JsonRead,
        }



from logging import Logger
from pyspark.sql import DataFrame


class PathWriter:
    def __init__(self, format_: str, logger: Logger):
        self.__format = format_
        self.__logger = logger

    def write(self, df: DataFrame, path: str, mode: str, partition_by: list = None, options: dict = None):
        self.__logger.info(
            f"Writing data to: {path}",
            extra={
                "mode": mode,
                "partition_by": partition_by,
                "options": options,
            },
        )

        data_frame_writer = df.write.mode(mode)

        if partition_by:
            data_frame_writer = data_frame_writer.partitionBy(partition_by)

        if options:
            data_frame_writer = data_frame_writer.options(**options)

        data_frame_writer.format(self.__format).save(path)

        self.__logger.info(f"Data successfully written to: {path}")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pysparkbundle',
 'pysparkbundle.csv',
 'pysparkbundle.csv.lineage',
 'pysparkbundle.delta',
 'pysparkbundle.jdbc',
 'pysparkbundle.json',
 'pysparkbundle.json.lineage',
 'pysparkbundle.lineage',
 'pysparkbundle.lineage.argument',
 'pysparkbundle.parquet',
 'pysparkbundle.read',
 'pysparkbundle.test',
 'pysparkbundle.write']

package_data = \
{'': ['*'], 'pysparkbundle': ['_config/*']}

install_requires = \
['daipe-core>=0.8.0a7,<0.9.0',
 'injecta>=0.10.0,<0.11.0',
 'pyfony-bundles>=0.4.0,<0.5.0']

entry_points = \
{'pyfony.bundle': ['create = pysparkbundle.PySparkBundle:PySparkBundle']}

setup_kwargs = {
    'name': 'pyspark-bundle',
    'version': '0.1.0a4',
    'description': 'PySpark bundle for the Daipe Framework',
    'long_description': '# PySpark bundle\n\nThis bundle contains PySpark related helper functions used by the [Daipe Framework](https://www.daipe.ai).  \n\n## Resources\n\n* [Documentation](https://docs.daipe.ai/)\n',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/daipe-ai/pyspark-bundle',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
